%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% MCDCTrain.m
% 
% ■新規実行
% 
% [IDX, SKP, OKP, FV, GV, XE, YE, loglik] = MCDCTrain(algorithm, ...
%                                                     xGrids, ...
%                                                     stateKernelGens, obsKernelGens, ...
%                                                     stateMeanFuncs, obsMeanFuncs, ...
%                                                     gridDimForGramMatrix, ...
%                                                     splineHandle, ...
%                                                     x0, u, z, N, J, K, ...
%                                                     aspect)
% 
% input:
%   algorithm: Algorithm オブジェクト（PSMC, PMCMC 等）
%   xGrids: 状態変数値のグリッドデータ（次元毎データのセル配列）
%   stateKernelGens: 状態変数次元毎のカーネル関数生成オブジェクト (KernelGenerator) セル配列
%   obsKernelGens: 観測変数次元毎のカーネル関数生成オブジェクト (KernelGenerator) セル配列
%   stateMeanFuncs: 状態変数次元毎の平均値関数セル配列
%   obsMeanFuncs: 観測変数次元毎の平均値関数セル配列
%   gridDimForGramMatrix: 
%   splineHandle: spline 補間関数生成関数ハンドル
%   x0: 状態変数の初期値
%   xaux: 状態変数に対する時刻ごとの制御入力 (グリッドに含める)
%   u: 状態変数に対する時刻ごとの制御入力
%   z: 観測値の時系列データ
%   N: Particle Filter 粒子数
%   J: Iteratrion 回数
%   K: 状態と観測の時刻差
%   aspect: Aspect オブジェクト
% 
% output:
%   IDX: Iteration に対応する、他の出力値（SKP, OKP, FV, GV, XE, YE, loglik）のインデックス番号
%   SKP: 状態変数カーネル関数パラメータ
%   OKP: 観測変数カーネル関数パラメータ
%   FV: 各グリッドにおける状態関数値
%   GV: 各グリッドにおける観測関数値
%   XE: 状態値時間推移
%   YE: 観測値時間推移
%   loglik: 尤度の推移
% 
% ■継続実行
% 
% [IDX, SKP, OKP, FV, GV, XE, YE, loglik] = MCDCTrain(matfile, aspect, Name, Value. ...)
% 
% input:
%   matfile: 前回実行時中間状態の .mat ファイル名
%   aspect: Aspect オブジェクト
%   Name, Value, ...: 前回実行時のパラメータを変更して継続実行する場合の、引数ペア
%     'N': Particle Filter 粒子数
% 
% output:
%   新規実行時と同じ
%
function [ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain(varargin)
  if isa(varargin{1}, 'Algorithm')
    algorithm = varargin{1};
    in = MCDCInput(varargin{2:15});
    out = MCDCOutput(in.J);
    aspect = varargin{16};
  elseif ischar(varargin{1})
    [algorithm, in, out] = MCDCMatFile.Load(varargin{1});
    aspect = varargin{2};
    overrideInput(in, varargin{:});
  end
  
  algorithm.Execute(in, out, aspect);
  IDX = out.IDX;
  SKP = out.SKP;
  OKP = out.OKP;
  FV = out.FV;
  GV = out.GV;
  XE = out.XE;
  YE = out.YE;
  loglik = out.loglik;
  
end

function overrideInput(in, varargin)
  n = 3;
  while (nargin >= (n + 1))
    key = varargin{n};
    if (strcmp(key, 'N'))
      in.N = varargin{n + 1};
    end
    n = n + 2;
  end
end
